import React from 'react';
import ReactDOM from 'react-dom';
import { range, scaleBand, arc } from 'd3';

// Production rules for the L-System.
const rules = {
  A: 'AB',
  B: 'AB'
};

const initialState = 'A';
const iterations = 12;
const L = str =>
  Array.from(str)
    .map(d => rules[d])
    .join('');
const iterate = i => (i ? L(iterate(i - 1)) : initialState);
const data = range(iterations).map(iterate);

// console.log(JSON.stringify(data, null, 2));
// Prints something like this:
// [
//   "A",
//   "ABA",
//   "ABABBBABA",
//   "ABABBBABABBBBBBBBBABABBBABA",
//   "ABABBBABABBBBBBBBBABABBBABABBBBBBBBBBBBBBBBBBBBBBBBBBBABABBBABABBBBBBBBBABABBBABA",
//   ...

const width = 960;
const height = 960;
const x = scaleBand().range([-Math.PI, Math.PI]);
const y = d3
  .scaleBand()
  .domain(d3.range(iterations))
  .rangeRound([0, height / 2]);
const markArc = arc().cornerRadius(5);
const arcs = data
  .map((str, j) => {
    x.domain(d3.range(str.length));
    return Array.from(str)
      .map((symbol, i) => ({
        symbol: symbol,
        innerRadius: y(j),
        outerRadius: y(j) + y.bandwidth() - 1,
        startAngle: x(i),
        endAngle: x(i) + x.bandwidth()
      }))
      .filter(d => d.symbol === 'A');
  })
  .reduce((a, b) => a.concat(b), []);

console.log(arcs);

const App = () => (
  <svg width={width} height={height}>
    <g transform={`translate(${width / 2},${height / 2})`}>
      {arcs.map(d => (
        <path d={markArc(d)} fill="black" />
      ))}
    </g>
  </svg>
);

ReactDOM.render(<App />, document.getElementById('root'));
